@@ -48,6 +48,16 @@ PYTHON_OPERATOR = {
     ">": ">"
 }
 
+MISSING_OPERATOR = {
+    "=": "is",
+    "!=": "is not"
+}
+
+T_MISSING_OPERATOR = {
+    "=": "ISNULL(",
+    "!=": "NOT ISNULL("
+}
+
 
 MAX_ARGS_LENGTH = 10
 
@@ -178,6 +188,29 @@ def filter_nodes(nodes_list, ids=None, subtree=True):
     return nodes
 
 
+def missing_branch(children):
+    """Checks if the missing values are assigned to a special branch
+
+    """
+    return any([child.predicate.missing for child in children])
+
+
+def none_value(children):
+    """Checks if the predicate has a None value
+
+    """
+    return any([child.predicate.value is None for child in children])
+
+
+def one_branch(children, input_data):
+    """Check if there's only one branch to be followed
+
+    """
+    missing = split(children) in input_data
+    return (missing or missing_branch(children)
+            or none_value(children))
+
+
 class Tree(object):
     """A tree-like predictive model.
 
@@ -318,9 +351,8 @@ class Tree(object):
                     prediction, instances = final_distribution.items()[0]
                     if instances == 1:
                         return (last_node.output, path, last_node.confidence,
-                                last_node.distribution, instances)      
-                # when there's more instantces, sort elements by their mean
-                # sort elements by their mean
+                                last_node.distribution, instances)               
+                # when there's more instances, sort elements by their mean
                 distribution = [list(element) for element in
                                 sorted(final_distribution.items(),
                                        key=lambda x: x[0])]
@@ -342,7 +374,7 @@ class Tree(object):
                         distribution, get_instances(distribution))
 
         else:
-            if self.children and split(self.children) in input_data:
+            if self.children:
                 for child in self.children:
                     if child.predicate.apply(input_data, self.fields):
                         path.append(child.predicate.to_rule(self.fields))
@@ -369,7 +401,7 @@ class Tree(object):
             return (merge_distributions({}, dict((x[0], x[1])
                                                  for x in self.distribution)),
                     self)
-        if split(self.children) in input_data:
+        if one_branch(self.children, input_data):
             for child in self.children:
                 if child.predicate.apply(input_data, self.fields):
                     new_rule = child.predicate.to_rule(self.fields)
@@ -437,7 +469,7 @@ class Tree(object):
             """
             if input_map:
                 if missing:
-                    return "not '%s' in data or data['%s']" % (field, field)
+                    return "data.get('%s')" % field
                 else:
                     return "data['%s']" % field
             return field
@@ -449,7 +481,12 @@ class Tree(object):
                                 subtree=subtree)
         if children:
             field = split(children)
-            if not self.fields[field]['slug'] in cmv:
+            has_missing_branch = (missing_branch(children) or
+                                  none_value(children))
+            # the missing is singled out as a special case only when there's
+            # no missing branch in the children list
+            if (not has_missing_branch and
+                not self.fields[field]['slug'] in cmv):
                 body += (u"%sif (%s is None):\n" %
                          (INDENT * depth,
                           map_data(self.fields[field]['slug'], True)))
@@ -463,32 +500,51 @@ class Tree(object):
                 cmv.append(self.fields[field]['slug'])
 
             for child in children:
-                optype = self.fields[child.predicate.field]['optype']
-                if optype == 'numeric' or optype == 'text':
+                field = child.predicate.field
+                pre_condition = u""
+                if has_missing_branch and child.predicate.value is not None:
+                    negation = u"" if child.predicate.missing else u" not"
+                    connection = u"or" if child.predicate.missing else u"and"
+                    pre_condition = (u"%s is%s None %s " % (
+                                     map_data(self.fields[field]['slug'],
+                                              True),
+                                     negation,
+                                     connection))
+                    if not child.predicate.missing:
+                        cmv.append(self.fields[field]['slug'])
+                optype = self.fields[field]['optype']
+                if (optype == 'numeric' or optype == 'text' or
+                    child.predicate.value is None):
                     value = child.predicate.value
                 else:
                     value = repr(child.predicate.value)
                 if optype == 'text':
                     body += (
-                        u"%sif (term_matches(%s, \"%s\", %s\"%s\") %s %s):\n" %
-                        (INDENT * depth,
-                         map_data(self.fields[child.predicate.field]['slug'],
+                        u"%sif (%sterm_matches(%s, \"%s\", %s\"%s\") %s %s):"
+                        u"\n" %
+                        (INDENT * depth, pre_condition,
+                         map_data(self.fields[field]['slug'],
                                   False),
-                         self.fields[child.predicate.field]['slug'],
+                         self.fields[field]['slug'],
                          ('u' if isinstance(child.predicate.term, unicode)
                           else ''),
                          child.predicate.term.replace("\"", "\\\""),
                          PYTHON_OPERATOR[child.predicate.operator],
                          value))
-                    term_analysis_fields.append((child.predicate.field,
+                    term_analysis_fields.append((field,
                                                  child.predicate.term))
                 else:
+                    operator = (MISSING_OPERATOR[child.predicate.operator] if
+                                child.predicate.value is None else
+                                PYTHON_OPERATOR[child.predicate.operator])
+                    if child.predicate.value is None:
+                        cmv.append(self.fields[field]['slug'])
                     body += (
-                        u"%sif (%s %s %s):\n" %
-                        (INDENT * depth,
-                         map_data(self.fields[child.predicate.field]['slug'],
+                        u"%sif (%s%s %s %s):\n" %
+                        (INDENT * depth, pre_condition,
+                         map_data(self.fields[field]['slug'],
                                   False),
-                         PYTHON_OPERATOR[child.predicate.operator],
+                         operator,
                          value))
                 next_level = child.python_body(depth + 1, cmv=cmv[:],
                                                input_map=input_map,
@@ -671,17 +727,23 @@ class Tree(object):
 
         if cmv is None:
             cmv = []
-        if conditions is None:
-            conditions = []
-            alternate = u"IF"
-        else:
+        if body:
             alternate = u"ELSEIF"
+        else:
+            if conditions is None:
+                conditions = []
+            alternate = u"IF"
 
         children = filter_nodes(self.children, ids=ids_path,
                                 subtree=subtree)
         if children:
             field = split(children)
-            if not self.fields[field]['name'] in cmv:
+            has_missing_branch = (missing_branch(children) or
+                                  none_value(children))
+            # the missing is singled out as a special case only when there's
+            # no missing branch in the children list
+            if (not has_missing_branch and
+                    not self.fields[field]['name'] in cmv):
                 conditions.append("ISNULL([%s])" % self.fields[field]['name'])
                 body += (u"%s %s THEN " %
                          (alternate, " AND ".join(conditions)))
@@ -695,17 +757,40 @@ class Tree(object):
                 del conditions[-1]
 
             for child in children:
+                pre_condition = u""
+                post_condition = u""
+                if has_missing_branch and child.predicate.value is not None:
+                    negation = u"" if child.predicate.missing else u"NOT "
+                    connection = u"OR" if child.predicate.missing else u"AND"
+                    pre_condition = (u"(%sISNULL([%s]) %s " % (
+                                     negation, self.fields[field]['name'],
+                                     connection))
+                    if not child.predicate.missing:
+                        cmv.append(self.fields[field]['name'])
+                    post_condition = u")"
                 optype = self.fields[child.predicate.field]['optype']
-                if optype == 'text':
+                if child.predicate.value is None:
+                    value = ""
+                elif optype == 'text':
                     return u""
-                if optype == 'numeric':
+                elif optype == 'numeric':
                     value = child.predicate.value
                 else:
                     value = repr(child.predicate.value)
-                conditions.append("[%s]%s%s" % (
+
+                operator = ("" if child.predicate.value is None else
+                            PYTHON_OPERATOR[child.predicate.operator])
+                if child.predicate.value is None:
+                    pre_condition = (
+                        T_MISSING_OPERATOR[child.predicate.operator])
+                    post_condition = ")"
+
+                conditions.append("%s[%s]%s%s%s" % (
+                    pre_condition,
                     self.fields[child.predicate.field]['name'],
-                    PYTHON_OPERATOR[child.predicate.operator],
-                    value))
+                    operator,
+                    value,
+                    post_condition))
                 body = child.tableau_body(body, conditions[:], cmv=cmv[:],
                                           ids_path=ids_path, subtree=subtree)
                 del conditions[-1]
