@@ -80,6 +80,20 @@ def wait_until_batch_centroid_status_code_is(step, code1, code2, secs):
     assert status['code'] == int(code1)
 
 
+@step(r'I wait until the batch anomaly score status code is either (\d) or (-\d) less than (\d+)')
+def wait_until_batch_anomaly_score_status_code_is(step, code1, code2, secs):
+    start = datetime.utcnow()
+    step.given('I get the batch anomaly score "{id}"'.format(id=world.batch_anomaly_score['resource']))
+    status = get_status(world.batch_anomaly_score)
+    while (status['code'] != int(code1) and
+           status['code'] != int(code2)):
+        time.sleep(3)
+        assert datetime.utcnow() - start < timedelta(seconds=int(secs))
+        step.given('I get the batch anomaly score "{id}"'.format(id=world.batch_anomaly_score['resource']))
+        status = get_status(world.batch_anomaly_score)
+    assert status['code'] == int(code1)
+
+
 @step(r'I wait until the batch prediction is ready less than (\d+)')
 def the_batch_prediction_is_finished_in_less_than(step, secs):
     wait_until_batch_prediction_status_code_is(step, FINISHED, FAULTY, secs)
@@ -88,6 +102,10 @@ def the_batch_prediction_is_finished_in_less_than(step, secs):
 def the_batch_centroid_is_finished_in_less_than(step, secs):
     wait_until_batch_centroid_status_code_is(step, FINISHED, FAULTY, secs)
 
+@step(r'I wait until the batch anomaly score is ready less than (\d+)')
+def the_batch_anomaly_score_is_finished_in_less_than(step, secs):
+    wait_until_batch_anomaly_score_status_code_is(step, FINISHED, FAULTY, secs)
+
 
 @step(r'I download the created predictions file to "(.*)"')
 def i_download_predictions_file(step, filename):
@@ -103,6 +121,13 @@ def i_download_centroid_file(step, filename):
     assert file_object is not None
     world.output = file_object
 
+@step(r'I download the created anomaly score file to "(.*)"')
+def i_download_anomaly_score_file(step, filename):
+    file_object = world.api.download_batch_anomaly_score(
+        world.batch_anomaly_score, filename=filename)
+    assert file_object is not None
+    world.output = file_object
+
 @step(r'the batch prediction file is like "(.*)"')
 def i_check_predictions(step, check_file):
     predictions_file = world.output
@@ -133,11 +158,18 @@ def i_check_predictions(step, check_file):
 def i_check_batch_centroid(step, check_file):
     i_check_predictions(step, check_file)
 
+@step(r'the batch anomaly score file is like "(.*)"')
+def i_check_batch_anomaly_score(step, check_file):
+    i_check_predictions(step, check_file)
 
 @step(r'I check the batch centroid is ok')
 def i_check_batch_centroid_is_ok(step):
     assert world.api.ok(world.batch_centroid)
 
+@step(r'I check the batch anomaly score is ok')
+def i_check_batch_anomaly_score_is_ok(step):
+    assert world.api.ok(world.batch_anomaly_score)
+
 
 @step(r'I create a batch centroid for the dataset$')
 def i_create_a_batch_prediction_with_cluster(step):
@@ -150,6 +182,17 @@ def i_create_a_batch_prediction_with_cluster(step):
     world.batch_centroid = resource['object']
     world.batch_centroids.append(resource['resource'])
 
+@step(r'I create a batch anomaly score$')
+def i_create_a_batch_prediction_with_anomaly(step):
+    dataset = world.dataset.get('resource')
+    anomaly = world.anomaly.get('resource')
+    resource = world.api.create_batch_anomaly_score(anomaly, dataset)
+    world.status = resource['code']
+    assert world.status == HTTP_CREATED
+    world.location = resource['location']
+    world.batch_anomaly_score = resource['object']
+    world.batch_anomaly_scores.append(resource['resource'])
+
 
 @step(r'I create a source from the batch prediction$')
 def i_create_a_source_from_batch_prediction(step):
